#set of functions, directly/indirectly involved, in oneChannelGUI filteringMenu
#oneChannelGUI  iqrFilter performs a filtering of gene and/or exon data based on iqr filtering from genefilter
#oneChannelGUI  intensityFilter performs a filtering of gene and/or exon data based intensity data and pOverA function from genefilter
#oneChannelGUI listFilter  performs a filtering of gene and/or exon data based on a list of probesets 
         #contains OpenAFile present in filemenu.R, written immediately after the OpenLargeFile
#oneChannelGUI IPAlistFilter  performs a filtering of gene and/or exon data based on a list of Entez Genes IDs
#oneChannelGUI ExportNormalizedExpressionValues1 present in filemenu.R
#oneChannelGUI intronicBg this function calculate the density distribution for exon (positive) and intron (negative controls) of the controls
               #and allows the user to define a background threshold to be used fgor filtering
#oneChannelGUI plierToZero this function changes very low log2 intesities values to 0, this is expecialy usefull for plier data               
#oneChannelGUI dabgFilter data are filtered on the basis of a p-value defining the probability of expression over background for a probe
#oneChannelGUI recoverUnfiltered data this function allow to recover ffrom an external file the unfiltered data
#oneChannelGUI reviqrFilter reverse IQR filtering for exon analysis. Removes strongly changing gene-level rpobe set that might 
#make more difficult the detection of alternative splcing events.
#oneChannelGUI biomartFilter filtering out all EGs which do not have at least 2 trasncripts associated on the basis of ensembl annotation
################################################################################
# x is an ExpressionSet, threshold is the IQR value to be applied
"iqrFilter"<-function(){
          #error if no data are loaded
          Try(whichArrayPlatform <- get("whichArrayPlatform",envir=affylmGUIenvironment))
          if(whichArrayPlatform ==""){
              Try(tkmessageBox(title="Filtering",message="No arrays have been loaded.	Please try New or Open from the File menu.",type="ok",icon="error"))
				    	Try(tkfocus(.affylmGUIglobals$ttMain))
              Try(return())
          }    
          #########################

            require(genefilter) || stop("library genefilter could not be found !")
           	##makning the menu for selecting IQR threshold
            Try(ttGetFilterMethod <- tktoplevel(.affylmGUIglobals$ttMain))
	      Try(tkwm.deiconify(ttGetFilterMethod))
            Try(tkgrab.set(ttGetFilterMethod))
            Try(tkfocus(ttGetFilterMethod))
            Try(tkwm.title(ttGetFilterMethod,"IQR Filtering Threshold"))
	
 	      Try(tkgrid(tklabel(ttGetFilterMethod,text="    ")))
	      Try(ttGetFilterMethodTcl <- tclVar("0.25"))
            Try(rbIQR.5 <- tkradiobutton(ttGetFilterMethod,text="IQR 0.5",variable=ttGetFilterMethodTcl,value=0.5,font=.affylmGUIglobals$affylmGUIfont2))
            Try(rbIQR.25<-tkradiobutton(ttGetFilterMethod,text="IQR 0.25",variable=ttGetFilterMethodTcl,value=0.25,font=.affylmGUIglobals$affylmGUIfont2))
            Try(rbIQR.1<-tkradiobutton(ttGetFilterMethod,text="IQR 0.1",variable=ttGetFilterMethodTcl,value=0.1,font=.affylmGUIglobals$affylmGUIfont2))
	      Try(tkgrid(tklabel(ttGetFilterMethod,text="    "),rbIQR.5))
	      Try(tkgrid(tklabel(ttGetFilterMethod,text="    "),rbIQR.25))
	      Try(tkgrid(tklabel(ttGetFilterMethod,text="    "),rbIQR.1))
	      Try(tkgrid.configure(rbIQR.5,rbIQR.25,rbIQR.1,columnspan=2,sticky="w"))
	      Try(tkgrid(tklabel(ttGetFilterMethod,text="    "),tklabel(ttGetFilterMethod,text="    ")))
	      Try(ReturnVal <- "")
	      Try(onCancel <- function() {Try(ReturnVal <<- "");Try(tkgrab.release(ttGetFilterMethod));Try(tkdestroy(ttGetFilterMethod));Try(tkfocus(.affylmGUIglobals$ttMain))})
	      Try(onOK <- function() {Try(ReturnVal <<- tclvalue(ttGetFilterMethodTcl));Try(tkgrab.release(ttGetFilterMethod));Try(tkdestroy(ttGetFilterMethod));Try(tkfocus(.affylmGUIglobals$ttMain))})
        Try(onHelp <- function() tkmessageBox(title="IQR filtering",
                 message="IQR filter remove invariant probe set \non the basis of their distribution in the arrays under analysis.
                           \nIQR=0.1 soft filter, i.e. few invariant probe sets are removed, \nIQR=0.25 intermediate filter, \nIQR=0.5 robust filter, i.e. most of the invariant probe sets are removed.",
                 icon="info"))
        Try(Help.but <- tkbutton(ttGetFilterMethod,text=" Help ",command=function()Try(onHelp()),font=.affylmGUIglobals$affylmGUIfont2))
	      Try(OK.but     <- tkbutton(ttGetFilterMethod,text="OK",command=onOK,font=.affylmGUIglobals$affylmGUIfont2))
	      Try(Cancel.but <- tkbutton(ttGetFilterMethod,text="Cancel",command=onCancel,font=.affylmGUIglobals$affylmGUIfont2))

	      Try(tkgrid(tklabel(ttGetFilterMethod,text="    "),OK.but,Cancel.but, Help.but,tklabel(ttGetFilterMethod,text="    ")))
	      Try(tkgrid.configure(OK.but,sticky="e"))
	      Try(tkgrid.configure(Cancel.but,sticky="w"))
	      Try(tkgrid.configure(Help.but,sticky="e"))
	      Try(tkgrid(tklabel(ttGetFilterMethod,text="    ")))

	      Try(tkbind(ttGetFilterMethod,"<Destroy>",function() {ReturnVal <- "";Try(tkgrab.release(ttGetFilterMethod));Try(tkfocus(.affylmGUIglobals$ttMain));}))
       Try(tkbind(OK.but, "<Return>",onOK))
       Try(tkbind(Cancel.but, "<Return>",onCancel))      
        Try(tkbind(Help.but, "<Return>",onCancel)) 
	      Try(tkwait.window(ttGetFilterMethod))
	      if(is.na(as.numeric(ReturnVal))){
                              Try(tkconfigure(.affylmGUIglobals$ttMain,cursor="arrow"))
                              return()
        } else{
        ##Filtering routine
        Try(whichArrayPlatform <- get("whichArrayPlatform", env=affylmGUIenvironment))
        Try(    
            if(whichArrayPlatform=="EXON"){
                     FilterMethod=paste("IQR filter", ReturnVal, sep=" ")
                     threshold= as.numeric(ReturnVal)
                     NormalizedAffyData.Available     <- get("NormalizedAffyData.Available" , envir=affylmGUIenvironment)
                     if(NormalizedAffyData.Available){
                            Try(tkconfigure(.affylmGUIglobals$ttMain,cursor="watch"))
                            x <- get("NormalizedAffyData",envir=affylmGUIenvironment)
                            iqr <- function(x){IQR(x)>threshold}
                            ff <- filterfun(iqr)
                            which <- genefilter(x, ff)
                            par(mfrow=c(1,2))
                            text<-paste("Unfiltered probes= ", dim(exprs(x))[1], sep="")
                            hist(as.matrix(exprs(x)), breaks=100, main=text)
                            text<-paste("Filtered probes= ", sum(which), " ",FilterMethod,sep="")
                            hist(as.matrix(exprs(x[which,])), breaks=100, main=text)
                            affylmGUIenvironment$NormalizedAffyData<-x[which,]
                            x.exon <- get("exonAffyData",envir=affylmGUIenvironment)
                            ####temporary saving unfiltered data
                            tempUnfiltered <- tempfile(pattern = "unfiltered", tmpdir = "")
                            tempUnfiltered <- sub( "\\\\","",tempUnfiltered)
                            tempUnfiltered <- sub( "/","",tempUnfiltered)
                            workingDir <- getwd()
                            workingDir <- sub( "/$","",workingDir)  
                            tempUnfiltered <- paste(workingDir, "/", tempUnfiltered, ".rda", sep="")
                            assign("unfilteredData.location", tempUnfiltered, envir=affylmGUIenvironment)
                            save(x, x.exon, file=tempUnfiltered)
                            ####temporary saving unfiltered data
                            tkdelete(.affylmGUIglobals$mainTree,"NormalizedAffyData.Status")
                            tkinsert(.affylmGUIglobals$mainTree,"end","NormalizedAffyData","NormalizedAffyData.Status" ,text=paste("Available (",FilterMethod,")",sep=""),font=.affylmGUIglobals$affylmGUIfontTree)
                            x <- x[which,]#subset of probe set to be used to extract exon data
                            x.lib <- affylmGUIenvironment$exprConsoleLibs$exon.library
                            x.lib.subset <- x.lib[which(x.lib[,1]%in%as.numeric(featureNames(x))),3]
                            x.lib.subset <- sapply(x.lib.subset ,function(x){strsplit(x, " ")})
                            x.lib.subset<-as.vector(unlist(x.lib.subset))
                            affylmGUIenvironment$exonAffyData <- x.exon[which(featureNames(x.exon)%in%x.lib.subset),]
                            Try(tkconfigure(.affylmGUIglobals$ttMain,cursor="arrow"))
                         }
           } else{   
                     FilterMethod=paste("IQR filter ", ReturnVal, sep=" ")
                     threshold= as.numeric(ReturnVal)
                     NormalizedAffyData.Available     <- get("NormalizedAffyData.Available" , envir=affylmGUIenvironment)
                     if(NormalizedAffyData.Available){
                            Try(tkconfigure(.affylmGUIglobals$ttMain,cursor="watch"))
                            x <- get("NormalizedAffyData",envir=affylmGUIenvironment)
                            iqr <- function(x){IQR(x)>threshold}
                            ff <- filterfun(iqr)
                            which <- genefilter(x, ff)
                            par(mfrow=c(1,2))
                            text<-paste("Unfiltered probes= ", dim(exprs(x))[1], sep="")
                            hist(as.matrix(exprs(x)), breaks=100, main=text)
                            text<-paste("Filtered probes= ", sum(which), " ",FilterMethod,sep="")
                            hist(as.matrix(exprs(x[which,])), breaks=100, main=text)
                            affylmGUIenvironment$NormalizedAffyData<-x[which,]
                            ####temporary saving unfiltered data
                            tempUnfiltered <- tempfile(pattern = "unfiltered", tmpdir = "")
                            tempUnfiltered <- sub( "\\\\","",tempUnfiltered)
                            tempUnfiltered <- sub( "/","",tempUnfiltered)
                            workingDir <- getwd()
                            workingDir <- sub( "/$","",workingDir)  
                            tempUnfiltered <- paste(workingDir, "/", tempUnfiltered, ".rda", sep="")
                            assign("unfilteredData.location", tempUnfiltered, envir=affylmGUIenvironment)
                            save(x, file=tempUnfiltered) 
                            ####temporary saving unfiltered data

                            tkdelete(.affylmGUIglobals$mainTree,"NormalizedAffyData.Status")
                            tkinsert(.affylmGUIglobals$mainTree,"end","NormalizedAffyData","NormalizedAffyData.Status" ,text=paste("Available (",FilterMethod,")",sep=""),font=.affylmGUIglobals$affylmGUIfontTree)
                            Try(tkconfigure(.affylmGUIglobals$ttMain,cursor="arrow"))
                         }
            }
     )
  }   
}
################################################################################

# x is an ExpressionSet, threshold is the min intenisty value to be applied, fraction is the % of array with above threshold
"intensityFilter"<-function(){
       require(genefilter) || stop("library genefilter could not be found !")
       #error if no data are loaded
       Try(whichArrayPlatform <- get("whichArrayPlatform",envir=affylmGUIenvironment))
       if(whichArrayPlatform ==""){
              Try(tkmessageBox(title="Filtering",message="No arrays have been loaded.	Please try New or Open from the File menu.",type="ok",icon="error"))
				    	Try(tkfocus(.affylmGUIglobals$ttMain))
              Try(return())
      }    
      #########################

       ##Filtering routine
       if(whichArrayPlatform=="EXON"){
         if(affylmGUIenvironment$intronicBg.available){  
           
           Try(intronicBg <- get("intronicBg", env=affylmGUIenvironment))    
           ##making the menu for selecting intensity threshold

           Try(ttIfDialog<-tktoplevel(.affylmGUIglobals$ttMain))
           Try(tkwm.deiconify(ttIfDialog))
           Try(tkgrab.set(ttIfDialog))
           Try(tkfocus(ttIfDialog))
           Try(tkwm.title(ttIfDialog,"Filtering Options"))
           Try(tkgrid(tklabel(ttIfDialog,text="    ")))

           Try(frame1 <- tkframe(ttIfDialog,relief="groove",borderwidth=2))
           Try(HowManyQuestion1 <- tklabel(frame1,text="Intensity threshold\ni.e. the selected intronic BG",font=.affylmGUIglobals$affylmGUIfont2))
           Try(tkgrid(HowManyQuestion1))
           Try(tkgrid.configure(HowManyQuestion1,columnspan=2,sticky="w"))
           Try(thresholdTcl <- tclVar(intronicBg))
           Try(I1.but  <- tkradiobutton(frame1,text=intronicBg, variable=thresholdTcl,value=intronicBg,font=.affylmGUIglobals$affylmGUIfont2))
           Try(tkgrid(I1.but,sticky="w"))
           Try(tkgrid.configure(HowManyQuestion1,I1.but,sticky="w"))

           Try(frame2 <- tkframe(ttIfDialog,relief="groove",borderwidth=2))
           Try(fractionLabel <- tklabel(frame2,text="% of arrays above threshold",font=.affylmGUIglobals$affylmGUIfont2))
           Try(tkgrid(fractionLabel,sticky="w"))
           Try(tkgrid.configure(fractionLabel,sticky="w"))
           Try(fractionTcl <- tclVar("1"))
           Try(F1.but <- tkradiobutton(frame2,text="10%",variable=fractionTcl,value="0.1",font=.affylmGUIglobals$affylmGUIfont2))
           Try(F2.but <- tkradiobutton(frame2,text="25%",variable=fractionTcl,value="0.25",font=.affylmGUIglobals$affylmGUIfont2))
           Try(F3.but <- tkradiobutton(frame2,text="50%",variable=fractionTcl,value="0.5",font=.affylmGUIglobals$affylmGUIfont2))
           Try(F4.but <- tkradiobutton(frame2,text="75%",variable=fractionTcl,value="0.75",font=.affylmGUIglobals$affylmGUIfont2))
           Try(F5.but <- tkradiobutton(frame2,text="100%",variable=fractionTcl,value="1",font=.affylmGUIglobals$affylmGUIfont2))

           Try(tkgrid(F1.but,sticky="w"))
           Try(tkgrid(F2.but,sticky="w"))
           Try(tkgrid(F3.but,sticky="w"))
           Try(tkgrid(F4.but,sticky="w"))
           Try(tkgrid(F5.but,sticky="w"))
           Try(tkgrid.configure(fractionLabel,F1.but,F2.but,F3.but,F4.but,F5.but,sticky="w"))

           Try(onOK <- function()
           {
                ReturnVal1 <- as.numeric(tclvalue(thresholdTcl))
                ReturnVal2 <- as.numeric(tclvalue(fractionTcl))
                Try(ReturnVal <<- paste(ReturnVal1, ReturnVal2, sep=";"))
                Try(tkgrab.release(ttIfDialog))
                Try(tkdestroy(ttIfDialog))
                Try(tkfocus(.affylmGUIglobals$ttMain))
           })

           Try(frame3 <- tkframe(ttIfDialog,borderwidth=2))
           Try(onCancel <- function() {Try(ReturnVal <<- ""); Try(tkgrab.release(ttIfDialog));Try(tkdestroy(ttIfDialog));Try(tkfocus(.affylmGUIglobals$ttMain))})
           Try(OK.but <-tkbutton(frame3,text="   OK   ",command=onOK,font=.affylmGUIglobals$affylmGUIfont2))
           Try(Cancel.but <-tkbutton(frame3,text=" Cancel ",command=onCancel,font=.affylmGUIglobals$affylmGUIfont2))

           Try(tkgrid(tklabel(frame3,text="    "),OK.but,Cancel.but,tklabel(frame3,text="    ")))

           Try(tkgrid(tklabel(ttIfDialog,text="    "),frame1,frame2,tklabel(ttIfDialog,text="  ")))
           Try(tkgrid(tklabel(ttIfDialog,text="    ")))
           Try(tkgrid(tklabel(ttIfDialog,text="    "),frame3,tklabel(ttIfDialog,text="  ")))
           Try(tkgrid(tklabel(ttIfDialog,text="    ")))
           Try(tkgrid.configure(frame1,frame3,sticky="w"))

           Try(tkfocus(ttIfDialog))
           Try(tkbind(ttIfDialog, "<Destroy>", function() {Try(tkgrab.release(ttIfDialog));Try(tkfocus(.affylmGUIglobals$ttMain));}))
           Try(tkwait.window(ttIfDialog))
           if(ReturnVal==""){return()}
                       tmp<-strsplit(ReturnVal, ";")
                       fraction= as.numeric(tmp[[1]][2])
                       threshold= intronicBg
                       tmp1 <- paste((fraction*100), "%", sep="")
                       tmp2 <- paste("threshold", intronicBg, sep=" ")
                       tmp3 <- paste(tmp1, tmp2, sep=" ")
                       FilterMethod=paste("I. filter: ", tmp3, sep=" ")
                       NormalizedAffyData.Available     <- get("NormalizedAffyData.Available" , envir=affylmGUIenvironment)
                       if(NormalizedAffyData.Available) {
                            Try(tkconfigure(.affylmGUIglobals$ttMain,cursor="watch"))
                            x <- get("NormalizedAffyData",envir=affylmGUIenvironment)
                            f1<-pOverA(fraction, threshold)
                            ff <- filterfun(f1)
                            which <- genefilter(x, ff)
                            par(mfrow=c(1,2))
                            text<-paste("Unfiltered probes= ", dim(exprs(x))[1], sep="")
                            hist(as.matrix(exprs(x)), main=text, breaks=100)
                            text<-paste("Filtered probes= ", sum(which), sep="")
                            hist(as.matrix(exprs(x[which,])), main=text, breaks=100)
                            affylmGUIenvironment$NormalizedAffyData<-x[which,]
                            x.exon <- get("exonAffyData",envir=affylmGUIenvironment)
                            ####temporary saving unfiltered data
                            tempUnfiltered <- tempfile(pattern = "unfiltered", tmpdir = "")
                            tempUnfiltered <- sub( "\\\\","",tempUnfiltered)
                            tempUnfiltered <- sub( "/","",tempUnfiltered)
                            workingDir <- getwd()
                            workingDir <- sub( "/$","",workingDir)  
                            tempUnfiltered <- paste(workingDir, "/", tempUnfiltered, ".rda", sep="")
                            assign("unfilteredData.location", tempUnfiltered, envir=affylmGUIenvironment)
                            save(x, x.exon, file=tempUnfiltered) 
                            ####temporary saving unfiltered data
                            tkdelete(.affylmGUIglobals$mainTree,"NormalizedAffyData.Status")
                            tkinsert(.affylmGUIglobals$mainTree,"end","NormalizedAffyData","NormalizedAffyData.Status" ,text=paste("Available (",FilterMethod,")",sep=""),font=.affylmGUIglobals$affylmGUIfontTree)
                            x.lib <- affylmGUIenvironment$exprConsoleLibs$exon.library
                            x.lib.subset <- x.lib[which(x.lib[,1]%in%as.numeric(featureNames(x))),3]
                            x.lib.subset <- sapply(x.lib.subset ,function(x){strsplit(x, " ")})
                            x.lib.subset<-as.vector(unlist(x.lib.subset))
                            affylmGUIenvironment$exonAffyData <- x.exon[which(featureNames(x.exon)%in%x.lib.subset),]
                            Try(tkconfigure(.affylmGUIglobals$ttMain,cursor="arrow"))
                       }
              } else{ Try(tkmessageBox(title="Intensity Filter for exon data",message=paste("To apply this filter you need to define an intensity threshold based on intronic data!\n Use the command oneChannelGUI: Set background threshold")))  
}        
           }else{
  
                       Try(ttIfDialog<-tktoplevel(.affylmGUIglobals$ttMain))
                       Try(tkwm.deiconify(ttIfDialog))
                       Try(tkgrab.set(ttIfDialog))
                       Try(tkfocus(ttIfDialog))
                       Try(tkwm.title(ttIfDialog,"Filtering Options"))
                       Try(tkgrid(tklabel(ttIfDialog,text="    ")))

                       Try(frame1 <- tkframe(ttIfDialog,relief="groove",borderwidth=2))
                       Try(HowManyQuestion1 <- tklabel(frame1,text="Intensity threshold",font=.affylmGUIglobals$affylmGUIfont2))
                       Try(tkgrid(HowManyQuestion1))
                       Try(tkgrid.configure(HowManyQuestion1,columnspan=2,sticky="w"))
                       Try(thresholdTcl <- tclVar("100"))
                       Try(I0.but  <- tkradiobutton(frame1,text="log2(1)=0",variable=thresholdTcl,value="1",font=.affylmGUIglobals$affylmGUIfont2))
                       Try(I1.but  <- tkradiobutton(frame1,text="log2(50)=5.64",variable=thresholdTcl,value="50",font=.affylmGUIglobals$affylmGUIfont2))
                       Try(I2.but  <- tkradiobutton(frame1,text="log2(100)=6.64",variable=thresholdTcl,value="100",font=.affylmGUIglobals$affylmGUIfont2))
                       Try(I3.but  <- tkradiobutton(frame1,text="log2(150)=7.22",variable=thresholdTcl,value="150",font=.affylmGUIglobals$affylmGUIfont2))
                       Try(I4.but  <- tkradiobutton(frame1,text="log2(200)=7.64",variable=thresholdTcl,value="200",font=.affylmGUIglobals$affylmGUIfont2))
                       Try(tkgrid(I0.but,sticky="w"))
                       Try(tkgrid(I1.but,sticky="w"))
                       Try(tkgrid(I2.but,sticky="w"))
                       Try(tkgrid(I3.but,sticky="w"))
                       Try(tkgrid(I4.but,sticky="w"))
                       Try(tkgrid.configure(HowManyQuestion1,I0.but,I1.but,I2.but,I3.but,I4.but,sticky="w"))

                       Try(frame2 <- tkframe(ttIfDialog,relief="groove",borderwidth=2))
                       Try(fractionLabel <- tklabel(frame2,text="% of arrays above threshold",font=.affylmGUIglobals$affylmGUIfont2))
                       Try(tkgrid(fractionLabel,sticky="w"))
                       Try(tkgrid.configure(fractionLabel,sticky="w"))
                       Try(fractionTcl <- tclVar("1"))
                       Try(F1.but <- tkradiobutton(frame2,text="10%",variable=fractionTcl,value="0.1",font=.affylmGUIglobals$affylmGUIfont2))
                       Try(F2.but <- tkradiobutton(frame2,text="25%",variable=fractionTcl,value="0.25",font=.affylmGUIglobals$affylmGUIfont2))
                       Try(F3.but <- tkradiobutton(frame2,text="50%",variable=fractionTcl,value="0.5",font=.affylmGUIglobals$affylmGUIfont2))
                       Try(F4.but <- tkradiobutton(frame2,text="75%",variable=fractionTcl,value="0.75",font=.affylmGUIglobals$affylmGUIfont2))
                       Try(F5.but <- tkradiobutton(frame2,text="100%",variable=fractionTcl,value="1",font=.affylmGUIglobals$affylmGUIfont2))

                       Try(tkgrid(F1.but,sticky="w"))
                       Try(tkgrid(F2.but,sticky="w"))
                       Try(tkgrid(F3.but,sticky="w"))
                       Try(tkgrid(F4.but,sticky="w"))
                       Try(tkgrid(F5.but,sticky="w"))
                       Try(tkgrid.configure(fractionLabel,F1.but,F2.but,F3.but,F4.but,F5.but,sticky="w"))

                       Try(onOK <- function()
                       {
                            ReturnVal1 <- as.numeric(tclvalue(thresholdTcl))
                            ReturnVal2 <- as.numeric(tclvalue(fractionTcl))
                            Try(ReturnVal <<- paste(ReturnVal1, ReturnVal2, sep=";"))
                            Try(tkgrab.release(ttIfDialog))
                            Try(tkdestroy(ttIfDialog))
                            Try(tkfocus(.affylmGUIglobals$ttMain))
                       })

                       Try(frame3 <- tkframe(ttIfDialog,borderwidth=2))
                       Try(onCancel <- function() {Try(ReturnVal <<- ""); Try(tkgrab.release(ttIfDialog));Try(tkdestroy(ttIfDialog));Try(tkfocus(.affylmGUIglobals$ttMain))})
                       Try(OK.but <-tkbutton(frame3,text="   OK   ",command=onOK,font=.affylmGUIglobals$affylmGUIfont2))
                       Try(Cancel.but <-tkbutton(frame3,text=" Cancel ",command=onCancel,font=.affylmGUIglobals$affylmGUIfont2))

                       Try(tkgrid(tklabel(frame3,text="    "),OK.but,Cancel.but,tklabel(frame3,text="    ")))

                       Try(tkgrid(tklabel(ttIfDialog,text="    "),frame1,frame2,tklabel(ttIfDialog,text="  ")))
                       Try(tkgrid(tklabel(ttIfDialog,text="    ")))
                       Try(tkgrid(tklabel(ttIfDialog,text="    "),frame3,tklabel(ttIfDialog,text="  ")))
                       Try(tkgrid(tklabel(ttIfDialog,text="    ")))
                       Try(tkgrid.configure(frame1,frame3,sticky="w"))

                       Try(tkfocus(ttIfDialog))
                       Try(tkbind(ttIfDialog, "<Destroy>", function() {Try(tkgrab.release(ttIfDialog));Try(tkfocus(.affylmGUIglobals$ttMain));}))
                       Try(tkwait.window(ttIfDialog))
                       if(ReturnVal==""){return()}
                       tmp<-strsplit(ReturnVal, ";")
                       fraction= as.numeric(tmp[[1]][2])
                       threshold= log2(as.numeric(tmp[[1]][1]))
                       tmp1 <- paste((fraction*100), "%", sep="")
                       tmp2 <- paste("threshold", as.numeric(tmp[[1]][1]), sep=" ")
                       tmp3 <- paste(tmp1, tmp2, sep=" ")
                       FilterMethod=paste("I. filter: ", tmp3, sep=" ")
                       NormalizedAffyData.Available     <- get("NormalizedAffyData.Available" , envir=affylmGUIenvironment)
                       if(NormalizedAffyData.Available) {
                            Try(tkconfigure(.affylmGUIglobals$ttMain,cursor="watch"))
                            x <- get("NormalizedAffyData",envir=affylmGUIenvironment)
                            ####temporary saving unfiltered data
                            tempUnfiltered <- tempfile(pattern = "unfiltered", tmpdir = "")
                            tempUnfiltered <- sub( "\\\\","",tempUnfiltered)
                            tempUnfiltered <- sub( "/","",tempUnfiltered)
                            workingDir <- getwd()
                            workingDir <- sub( "/$","",workingDir)  
                            tempUnfiltered <- paste(workingDir, "/", tempUnfiltered, ".rda", sep="")
                            assign("unfilteredData.location", tempUnfiltered, envir=affylmGUIenvironment)
                            save(x, file=tempUnfiltered) 
                            ####temporary saving unfiltered data

                            f1<-pOverA(fraction, threshold)
                            ff <- filterfun(f1)
                            which <- genefilter(x, ff)
                            par(mfrow=c(1,2))
                            text<-paste("Unfiltered probes= ", dim(exprs(x))[1], sep="")
                            hist(as.matrix(exprs(x)), main=text, breaks=100)
                            text<-paste("Filtered probes= ", sum(which), sep="")
                            hist(as.matrix(exprs(x[which,])), main=text, breaks=100)
                            affylmGUIenvironment$NormalizedAffyData<-x[which,]
                            tkdelete(.affylmGUIglobals$mainTree,"NormalizedAffyData.Status")
                            tkinsert(.affylmGUIglobals$mainTree,"end","NormalizedAffyData","NormalizedAffyData.Status" ,text=paste("Available (",FilterMethod,")",sep=""),font=.affylmGUIglobals$affylmGUIfontTree)
                            Try(tkconfigure(.affylmGUIglobals$ttMain,cursor="arrow"))           
                     }
             }
}
################################################################################

# Expression data are filtered on the basis of a list of affy ids
#performs filter also on exon data
"listFilter"<-function(){
          #error if no data are loaded
          Try(whichArrayPlatform <- get("whichArrayPlatform",envir=affylmGUIenvironment))
          if(whichArrayPlatform ==""){
              Try(tkmessageBox(title="Filtering",message="No arrays have been loaded.	Please try New or Open from the File menu.",type="ok",icon="error"))
				    	Try(tkfocus(.affylmGUIglobals$ttMain))
              Try(return())
          }    
          #########################
 
        Try(tkmessageBox(title="Filtering by list",message="Open the file containing the list of probe sets to be used for filtering"))
        Try(whichArrayPlatform <- get("whichArrayPlatform", env=affylmGUIenvironment))
        if(whichArrayPlatform=="AFFY" || whichArrayPlatform=="LARGE" || whichArrayPlatform=="ILLU") {
            OpenAFile <- function(FileName)
            {
               Try(tempFileName <- tclvalue(tkgetOpenFile(filetypes="{{Text Files} {.txt}} {{All files} *}")))
               if(!nchar(tempFileName))
               {
                  tkfocus(.affylmGUIglobals$ttMain)
                  return()
               }
               mySel<-read.table(tempFileName, sep="\t", header=F, as.is=TRUE)
               if(dim(mySel)[2]==1){
                    Try(tkconfigure(.affylmGUIglobals$ttMain,cursor="watch"))
                    Try(x <- get("NormalizedAffyData", envir= affylmGUIenvironment))
                            ####temporary saving unfiltered data
                            tempUnfiltered <- tempfile(pattern = "unfiltered", tmpdir = "")
                            tempUnfiltered <- sub( "\\\\","",tempUnfiltered)
                            tempUnfiltered <- sub( "/","",tempUnfiltered)
                            workingDir <- getwd()
                            workingDir <- sub( "/$","",workingDir)  
                            tempUnfiltered <- paste(workingDir, "/", tempUnfiltered, ".rda", sep="")
                    Try(assign("unfilteredData.location", tempUnfiltered, envir=affylmGUIenvironment))
                    Try(save(x, file=tempUnfiltered)) 
                    ####temporary saving unfiltered data

                    affylmGUIenvironment$NormalizedAffyData<-affylmGUIenvironment$NormalizedAffyData[which(featureNames(affylmGUIenvironment$NormalizedAffyData)%in%as.character(mySel[,1])),]
                    Try(NormalizedAffyData.Available     <- get("NormalizedAffyData.Available" , envir=affylmGUIenvironment))
                    Try(if(NormalizedAffyData.Available){
                                Try(tkdelete(.affylmGUIglobals$mainTree,"NormalizedAffyData.Status"))
                                Try(tkinsert(.affylmGUIglobals$mainTree,"end","NormalizedAffyData","NormalizedAffyData.Status" ,text="Available list filtered",font=.affylmGUIglobals$affylmGUIfontTree))
                         })
               Try(tkconfigure(.affylmGUIglobals$ttMain,cursor="arrow"))
               Try(tkmessageBox(title="Filtering by list",message=paste("Normalized Affy Data data is now made of ", dim(exprs(affylmGUIenvironment$NormalizedAffyData))[1]," probe sets",sep="")))

               } else Try(tkmessageBox(title="Filtering by list",message="Filter file should contain\nonly a column with\nAffymetrix ids", type="ok", icon="error"))  

            }
            OpenAFile()
      }
      if(whichArrayPlatform=="EXON") {
            OpenAFile <- function(FileName)
            {
               Try(tempFileName <- tclvalue(tkgetOpenFile(filetypes="{{Text Files} {.txt}} {{All files} *}")))
               if(!nchar(tempFileName))
               {
                  tkfocus(.affylmGUIglobals$ttMain)
                  return()
               }
               mySel<-read.table(tempFileName, sep="\t", header=F, as.is=TRUE)
               if(dim(mySel)[2]==1){
                    
                    Try(x <- get("NormalizedAffyData", envir= affylmGUIenvironment))
                    Try(x.exon <- get("exonAffyData",envir=affylmGUIenvironment))
                            ####temporary saving unfiltered data
                            tempUnfiltered <- tempfile(pattern = "unfiltered", tmpdir = "")
                            tempUnfiltered <- sub( "\\\\","",tempUnfiltered)
                            tempUnfiltered <- sub( "/","",tempUnfiltered)
                            workingDir <- getwd()
                            workingDir <- sub( "/$","",workingDir)  
                            tempUnfiltered <- paste(workingDir, "/", tempUnfiltered, ".rda", sep="")
                    Try(assign("unfilteredData.location", tempUnfiltered, envir=affylmGUIenvironment))
                    Try(save(x, x.exon, file=tempUnfiltered)) 
                    ####temporary saving unfiltered data

                    affylmGUIenvironment$NormalizedAffyData<-affylmGUIenvironment$NormalizedAffyData[which(featureNames(affylmGUIenvironment$NormalizedAffyData)%in%as.character(mySel[,1])),]
                    Try(NormalizedAffyData.Available     <- get("NormalizedAffyData.Available" , envir=affylmGUIenvironment))

                    Try(x.lib <- affylmGUIenvironment$exprConsoleLibs$exon.library)
                    Try(x.lib.subset <- x.lib[which(x.lib[,1]%in%featureNames(affylmGUIenvironment$NormalizedAffyData)),3]) #in the list obtained from NetAffx there are all the probe set ids and I have to subset only those related to the exon library loaded with the data set
                    Try(x.lib.subset <- sapply(x.lib.subset ,function(x){strsplit(x, " ")}))
                    Try(x.lib.subset<-as.vector(unlist(x.lib.subset)))
                    Try(affylmGUIenvironment$exonAffyData <- x.exon[which(featureNames(x.exon)%in%x.lib.subset),])


                    Try(if(NormalizedAffyData.Available){
                                Try(tkdelete(.affylmGUIglobals$mainTree,"NormalizedAffyData.Status"))
                                Try(tkinsert(.affylmGUIglobals$mainTree,"end","NormalizedAffyData","NormalizedAffyData.Status" ,text="Available list filtered",font=.affylmGUIglobals$affylmGUIfontTree))
                         })
               Try(tkmessageBox(title="Filtering by list",message=paste("Normalized Affy Data data is now made of ", dim(exprs(affylmGUIenvironment$NormalizedAffyData))[1]," probe sets",sep="")))

               } else Try(tkmessageBox(title="Filtering by list",message="Filter file should contain\nonly a column with\nAffymetrix ids", type="ok", icon="error"))  

            }
            OpenAFile()
      }

}
################################################################################

# Expression data are filtered on the basis of a list exported by IPA Ingenuity
"IPAlistFilter"<-function(){
      require(annotate) || stop("library annotate could not be found !")
     #error if no data are loaded
     Try(whichArrayPlatform <- get("whichArrayPlatform",envir=affylmGUIenvironment))
     if(whichArrayPlatform ==""){
              Try(tkmessageBox(title="Filtering",message="No arrays have been loaded.	Please try New or Open from the File menu.",type="ok",icon="error"))
				    	Try(tkfocus(.affylmGUIglobals$ttMain))
              Try(return())
    }    
    #########################

    # getDataEnv <- function(name, lib) {
    #    	            get(paste(lib, name, sep = ""), mode = "environment")
    # }
     Try(NormalizedAffyData.Available <- get("NormalizedAffyData.Available", env=affylmGUIenvironment))
     if((whichArrayPlatform=="AFFY" & NormalizedAffyData.Available)|| (whichArrayPlatform=="LARGE" & NormalizedAffyData.Available)) {
            Try(x <- get("NormalizedAffyData", env=affylmGUIenvironment))
                            ####temporary saving unfiltered data
                            tempUnfiltered <- tempfile(pattern = "unfiltered", tmpdir = "")
                            tempUnfiltered <- sub( "\\\\","",tempUnfiltered)
                            tempUnfiltered <- sub( "/","",tempUnfiltered)
                            workingDir <- getwd()
                            workingDir <- sub( "/$","",workingDir)  
                            tempUnfiltered <- paste(workingDir, "/", tempUnfiltered, ".rda", sep="")
                    Try(assign("unfilteredData.location", tempUnfiltered, envir=affylmGUIenvironment))
                    Try(save(x, file=tempUnfiltered)) 
                    ####temporary saving unfiltered data
            Try(lib <- .annotation(x))
            if(length(as.character(unlist(strsplit(lib, "\\.")))) < 2 ){
              lib <- paste(lib, ".db", sep="")     
            }   
            require(lib, character.only = TRUE) || {
                             Try(tkmessageBox(title="Filtering by Entrez Gene list",message=paste("need data package: ", lib, sep=" ")))
                             stop("need data package: ", lib)
            }
            Try(tkmessageBox(title="Filtering by EG list",message="Open the file containing a set of Entrez Gene IDs"))
            Try(tempFileName <- tclvalue(tkgetOpenFile(filetypes="{{Text Files} {.txt}} {{All files} *}")))
               if(!nchar(tempFileName))
               {
                  tkfocus(.affylmGUIglobals$ttMain)
                  return()
               }
               mySel<-read.table(tempFileName, sep="\t", header=F, as.is=TRUE, na.string="", quote="")
               if(dim(mySel)[2]==1){
                    Try(tkconfigure(.affylmGUIglobals$ttMain,cursor="watch"))
                     Try(annlib1 <- as.character(unlist(strsplit(lib, ".db"))))
                     Try(gNsel <- lookUp(featureNames(x), annlib1, "ENTREZID"))
                     Try(sub.gNsel <- gNsel[which(gNsel %in% as.character(mySel[,1]))])
                     x <- x[which(featureNames(x)%in%names(sub.gNsel)),]
                     assign("NormalizedAffyData", x, env=affylmGUIenvironment)
                     Try(tkdelete(.affylmGUIglobals$mainTree,"NormalizedAffyData.Status"))
                     Try(tkinsert(.affylmGUIglobals$mainTree,"end","NormalizedAffyData","NormalizedAffyData.Status" ,text="Available EG list filtered",font=.affylmGUIglobals$affylmGUIfontTree))
                     Try(tkconfigure(.affylmGUIglobals$ttMain,cursor="arrow"))
                     Try(tkmessageBox(title="Filtering by EG list",message=paste("Normalized Affy Data data is now made of ", dim(exprs(affylmGUIenvironment$NormalizedAffyData))[1]," probe sets",sep="")))

               } else Try(tkmessageBox(title="Filtering by EG list",message="Filter file should contain only an column of Entrez genes without header", type="ok", icon="error"))  


    } else if (whichArrayPlatform=="EXON" & NormalizedAffyData.Available){
            Try(tkmessageBox(title="Filtering by EG list",message="Open the file containing a set of Entrez Gene IDs"))
            Try(tempFileName <- tclvalue(tkgetOpenFile(filetypes="{{Text Files} {.txt}} {{All files} *}")))
               if(!nchar(tempFileName))
               {
                  tkfocus(.affylmGUIglobals$ttMain)
                  return()
               }
               mySel<-read.table(tempFileName, sep="\t", header=F, as.is=TRUE, na.string="", quote="")
               if(dim(mySel)[2]==1){
                    Try(tkconfigure(.affylmGUIglobals$ttMain,cursor="watch"))
                    Try(x <- get("NormalizedAffyData", env=affylmGUIenvironment))
                    Try(x.exon <- get("exonAffyData",envir=affylmGUIenvironment))
                            ####temporary saving unfiltered data
                            tempUnfiltered <- tempfile(pattern = "unfiltered", tmpdir = "")
                            tempUnfiltered <- sub( "\\\\","",tempUnfiltered)
                            tempUnfiltered <- sub( "/","",tempUnfiltered)
                            workingDir <- getwd()
                            workingDir <- sub( "/$","",workingDir)  
                            tempUnfiltered <- paste(workingDir, "/", tempUnfiltered, ".rda", sep="")
                    Try(assign("unfilteredData.location", tempUnfiltered, envir=affylmGUIenvironment))
                    Try(save(x, x.exon, file=tempUnfiltered)) 
                    ####temporary saving unfiltered data
                   
                    Try(whichLib <- get("whichLib",envir=affylmGUIenvironment))
                    Try(annlibloc <- grep(paste("^",whichLib[[1]][1],sep=""), as.vector(unlist(data(package="oneChannelGUI"))), ignore.case = T))
                    Try(if(whichLib[[1]][1]=="HuEx"){
                            #require(humanLLMappings) || stop("library humanLLMappings could not be found !")
                            #lib <- "humanLLMappings"
                            require(org.Hs.eg.db, quietly = TRUE) || stop("\nneed data package: org.Hs.eg.db\n")
                            lib = "org.Hs.eg.db"
                    })
                    Try(if(whichLib[[1]][1]=="MoEx"){
                            #require(mouseLLMappings) || stop("library mouseLLMappings could not be found !")
                            #lib <- "mouseLLMappings"
                            require(org.Mm.eg.db, quietly = TRUE) || stop("\nneed data package: org.Mm.eg.db\n")
                            lib = "org.Mm.eg.db"
                   })
                    Try(if(whichLib[[1]][1]=="RaEx"){
                            #require(ratLLMappings) || stop("library ratLLMappings could not be found !")
                            #lib <- "ratLLMappings"
                            require(org.Rn.eg.db, quietly = TRUE) || stop("\nneed data package: org.Rn.eg.db\n")
                            lib = "org.Rn.eg.db"
                   })
                    Try(annlibname <- as.vector(unlist(data(package="oneChannelGUI")))[annlibloc])
                    Try(data(list=annlibname,package="oneChannelGUI"))
                    Try(exonannlib <- get(annlibname,envir=.GlobalEnv))
                    Try(myrefseq <- as.character(exonannlib[which(as.character(exonannlib[,1])%in%featureNames(x)),2]))#selecting the refseq associated to the gene probesetids
                    #Try(gNacc <- as.character(unlist(mget(as.character(mySel[,1]), env=getDataEnv("LL2ACCNUM", lib), ifnotfound=NA))))#select refseq associated to EG            
                    Try(gNacc <- as.character(unlist(lookUp(as.character(mySel[,1]), lib, "ACCNUM"))))
                    Try(sub.gNacc <- intersect(gNacc,myrefseq))
                    Try(sub.probeset <- as.character(exonannlib[which(as.character(exonannlib[,2])%in%sub.gNacc),1]))
                    Try(x <- x[which(featureNames(x)%in%sub.probeset),])
     
                    Try(x.lib <- affylmGUIenvironment$exprConsoleLibs$exon.library)
                    Try(x.lib.subset <- x.lib[which(x.lib[,1]%in%as.numeric(featureNames(x))),3])
                    Try(x.lib.subset <- sapply(x.lib.subset ,function(x){strsplit(x, " ")}))
                    Try(x.lib.subset<-as.vector(unlist(x.lib.subset)))
                    Try(x.exon <- x.exon[which(featureNames(x.exon)%in%x.lib.subset),])
     
                    Try(assign("NormalizedAffyData", x, envir=affylmGUIenvironment))
                    Try(assign("exonAffyData", x.exon, envir=affylmGUIenvironment))
                    Try(tkdelete(.affylmGUIglobals$mainTree,"NormalizedAffyData.Status"))
                    Try(tkinsert(.affylmGUIglobals$mainTree,"end","NormalizedAffyData","NormalizedAffyData.Status" ,text="Available EG list filtered",font=.affylmGUIglobals$affylmGUIfontTree))
                    Try(tkdelete(.affylmGUIglobals$mainTree,"ExonArrays.Status"))
                    Try(tkinsert(.affylmGUIglobals$mainTree,"end","ExonArrays","ExonArrays.Status" ,text="Available EG list filtered",font=.affylmGUIglobals$affylmGUIfontTree))
                    Try(tkconfigure(.affylmGUIglobals$ttMain,cursor="arrow"))
                    Try(tkmessageBox(title="Filtering by EG list",message=paste("Normalized gene level data is now made of ", dim(exprs(x))[1]," probe sets","\nNormalized exon level data is now made of ", dim(exprs(x.exon))[1]," probe sets",sep="")))

               } else Try(tkmessageBox(title="Filtering by EG list",message="Filter file should contain only an column of Entrez genes without header", type="ok", icon="error"))  
    
    }

}
################################################################################

#this function calculate the density distribution for exon (positive) and intron (negative controls) of the controls
#and allows the user to define a background threshold to be used fgor filtering
"intronicBg" <- function(){
     #error if no data are loaded
     Try(whichArrayPlatform <- get("whichArrayPlatform",envir=affylmGUIenvironment))
     if(whichArrayPlatform ==""){
              Try(tkmessageBox(title="Filtering",message="No arrays have been loaded.	Please try New or Open from the File menu.",type="ok",icon="error"))
				    	Try(tkfocus(.affylmGUIglobals$ttMain))
              Try(return())
    } else if(whichArrayPlatform!="EXON"){
              Try(tkmessageBox(title="Filtering",message="This filter routine apply only to EXON arrays.",type="ok",icon="error"))
				    	Try(tkfocus(.affylmGUIglobals$ttMain))
              Try(return())
    
    }    
    #########################
      
      Try( ctrl <- affylmGUIenvironment$exprConsoleLibs$controls)#get control info
      Try( exonAffyData <- get("exonAffyData",envir=affylmGUIenvironment) )
      
      #check if ctrl aare still available and not removed by other filters
      Try(test <- intersect(featureNames(exonAffyData),ctrl[,1]))
      if(length(ctrl[,1]) != length(test)){
            Try(tkmessageBox(title="Intronic BG",message="The EXON and INTRON data for Housekeeping genes\nwere removed by some other filtering procedure.\nThis method can be applied only to the unfiltered data set.",type="ok",icon="error"))
	    	    Try(tkfocus(.affylmGUIglobals$ttMain))
             Try(return()) 
      }
      ###################################
      
      
      Try(exonAffyData.neg <- exonAffyData[which(featureNames(exonAffyData)%in%ctrl[which(ctrl[,2]=="neg_control"),1]),])
      Try(exonAffyData.pos <- exonAffyData[which(featureNames(exonAffyData)%in%ctrl[which(ctrl[,2]=="pos_control"),1]),])
      Try(plotDensity(as.matrix(exprs(exonAffyData.neg))))
      Try(legend(min(plotDensity(as.matrix(exprs(exonAffyData.neg)))$all.x),max(plotDensity(as.matrix(exprs(exonAffyData.neg)))$all.y), legend=c("Introns","Exons"), col=c("black","red"), pch=19))
      Try(plotDensity(as.matrix(exprs(exonAffyData.neg)), col="black", add=T))
      Try(plotDensity(as.matrix(exprs(exonAffyData.pos)), col="red", add=T))

      #selecting the BG threshold
                 Try(ttIntronicBg<-tktoplevel(.affylmGUIglobals$ttMain))
                 Try(tkwm.deiconify(ttIntronicBg))
                 Try(tkgrab.set(ttIntronicBg))
                 Try(tkfocus(ttIntronicBg))
                 Try(tkwm.title(ttIntronicBg,"Defining the BG threshold for filtering"))
                 Try(tkgrid(tklabel(ttIntronicBg,text="    ")))
                 Try(IntronicBgnum <- "6.64")
                 Try(Local.IntronicBg <- tclVar(init=IntronicBgnum))
                 Try(entry.IntronicBg <-tkentry(ttIntronicBg,width="4",font=.affylmGUIglobals$affylmGUIfont2,textvariable=Local.IntronicBg,bg="white"))
                 Try(tkgrid(tklabel(ttIntronicBg,text="Please enter a BG threshold.",font=.affylmGUIglobals$affylmGUIfont2)))
                 Try(tkgrid(entry.IntronicBg))
                 onOK <- function()
                 {
                     Try(IntronicBgnum <- as.numeric(tclvalue(Local.IntronicBg)))
                     Try(assign("intronicBg", as.numeric(tclvalue(Local.IntronicBg)),affylmGUIenvironment))
			               Try(assign("intronicBg.available", TRUE,affylmGUIenvironment))
 #                    Try(tclvalue(.affylmGUIglobals$IntronicBgTcl) <- IntronicBgnum)
                     Try(tkgrab.release(ttIntronicBg));Try(tkdestroy(ttIntronicBg));Try(tkfocus(.affylmGUIglobals$ttMain))                        
                 }
                 Try(OK.but <-tkbutton(ttIntronicBg,text="   OK   ",command=onOK,font=.affylmGUIglobals$affylmGUIfont2))
                 Try(tkgrid(tklabel(ttIntronicBg,text="    ")))
                 Try(tkgrid(OK.but))
                 Try(tkgrid.configure(OK.but))
                 Try(tkgrid(tklabel(ttIntronicBg,text="       ")))
                 Try(tkfocus(entry.IntronicBg))
                 Try(tkbind(entry.IntronicBg, "<Return>",onOK))
                 Try(tkbind(ttIntronicBg, "<Destroy>", function(){Try(tkgrab.release(ttIntronicBg));Try(tkfocus(.affylmGUIglobals$ttMain));return(0)}))
                 Try(tkwait.window(ttIntronicBg))
                 Try(tkfocus(.affylmGUIglobals$ttMain))
                 Try(IntronicBgnum <- get("intronicBg", env=affylmGUIenvironment))
                 Try(abline(v=as.numeric(IntronicBgnum), lty=2))
                 Try(tkmessageBox(title="BG threshold",message="The selected filtering threshold is shown by a dashed line in the main R window",type="ok",icon="info"))
}
################################################################################

#this function change very low log2 intesities values to 0 this is expecialy usefull for plier data
"plierToZero"<-function(){
           #error if no data are loaded
           Try(whichArrayPlatform <- get("whichArrayPlatform",envir=affylmGUIenvironment))
           if(whichArrayPlatform ==""){
              Try(tkmessageBox(title="Filtering",message="No arrays have been loaded.	Please try New or Open from the File menu.",type="ok",icon="error"))
				    	Try(tkfocus(.affylmGUIglobals$ttMain))
              Try(return())
           }    
           #########################

            Try(tkconfigure(.affylmGUIglobals$ttMain,cursor="watch"))
            Try(NormalizedAffyData.Available <- get("NormalizedAffyData.Available", env=affylmGUIenvironment))
            Try(exonAffyData.Available <- get("exonAffyData.Available", env=affylmGUIenvironment))
            
            if(NormalizedAffyData.Available & exonAffyData.Available){
               Try(NormalizedAffyData <- get("NormalizedAffyData", env=affylmGUIenvironment))
               Try(tmp <- esApply(NormalizedAffyData, 1, function(x){
                                                              x[which(x < 1)] <- 0
                                                              return(x)}
                         )
               )
               Try(exprs(NormalizedAffyData) <- as.matrix(t(tmp)))
               Try(assign("NormalizedAffyData", NormalizedAffyData, env=affylmGUIenvironment))
               Try(exonAffyData <- get("exonAffyData", env=affylmGUIenvironment))
               Try(tmp <- esApply(exonAffyData, 1, function(x){
                                                              x[which(x < 1)] <- 0
                                                              return(x)}
                         )
               )
               Try(exprs(exonAffyData) <- as.matrix(t(tmp)))
                Try(assign("exonAffyData", exonAffyData, env=affylmGUIenvironment))
                Try(tkmessageBox(title="Plier very low values",message="Log2 intensity values below 1 were converted to 0"))
            } else{
                       Try(tkmessageBox(title="Plier very low values",message="These are not exon array data")) 
            }
            Try(tkconfigure(.affylmGUIglobals$ttMain,cursor="arrow"))          
}
################################################################################

# Expression data are filtered on the basis of a list exported by IPA Ingenuity
"dabgFilter"<-function(){
     require(genefilter) || stop("library genefilter could not be found !")
     #error if no data are loaded
     Try(whichArrayPlatform <- get("whichArrayPlatform",envir=affylmGUIenvironment))
     if(whichArrayPlatform ==""){
              Try(tkmessageBox(title="Filtering",message="No arrays have been loaded.	Please try New or Open from the File menu.",type="ok",icon="error"))
				    	Try(tkfocus(.affylmGUIglobals$ttMain))
              Try(return())
    }    
    #########################

     Try(tkconfigure(.affylmGUIglobals$ttMain,cursor="watch"))
     Try(dabg.Available <- get("dabg.Available", env=affylmGUIenvironment))
     Try(exonAffyData.Available <- get("exonAffyData.Available", env=affylmGUIenvironment))
     if(dabg.Available & exonAffyData.Available){
                       Try(ttIfDialog<-tktoplevel(.affylmGUIglobals$ttMain))
                       Try(tkwm.deiconify(ttIfDialog))
                       Try(tkgrab.set(ttIfDialog))
                       Try(tkfocus(ttIfDialog))
                       Try(tkwm.title(ttIfDialog,"Filtering Options"))
                       Try(tkgrid(tklabel(ttIfDialog,text="    ")))

                       Try(frame1 <- tkframe(ttIfDialog,relief="groove",borderwidth=2))
                       Try(HowManyQuestion1 <- tklabel(frame1,text="DABG p-value threshold",font=.affylmGUIglobals$affylmGUIfont2))
                       Try(tkgrid(HowManyQuestion1))
                       Try(tkgrid.configure(HowManyQuestion1,columnspan=2,sticky="w"))
                       Try(thresholdTcl <- tclVar("0.05"))
                       Try(I1.but  <- tkradiobutton(frame1,text="0.05",variable=thresholdTcl,value="0.05",font=.affylmGUIglobals$affylmGUIfont2))
                       Try(I2.but  <- tkradiobutton(frame1,text="0.01",variable=thresholdTcl,value="0.01",font=.affylmGUIglobals$affylmGUIfont2))
                       Try(I3.but  <- tkradiobutton(frame1,text="0.005",variable=thresholdTcl,value="0.005",font=.affylmGUIglobals$affylmGUIfont2))
                       Try(I4.but  <- tkradiobutton(frame1,text="0.001",variable=thresholdTcl,value="0.001",font=.affylmGUIglobals$affylmGUIfont2))

                       Try(tkgrid(I1.but,sticky="w"))
                       Try(tkgrid(I2.but,sticky="w"))
                       Try(tkgrid(I3.but,sticky="w"))
                       Try(tkgrid(I4.but,sticky="w"))
                       Try(tkgrid.configure(HowManyQuestion1,I1.but,I2.but,I3.but,I4.but,sticky="w"))

                       Try(frame2 <- tkframe(ttIfDialog,relief="groove",borderwidth=2))
                       Try(fractionLabel <- tklabel(frame2,text="% of arrays below threshold",font=.affylmGUIglobals$affylmGUIfont2))
                       Try(tkgrid(fractionLabel,sticky="w"))
                       Try(tkgrid.configure(fractionLabel,sticky="w"))
                       Try(fractionTcl <- tclVar("0.5"))
                       Try(F1.but <- tkradiobutton(frame2,text="100%",variable=fractionTcl,value="1",font=.affylmGUIglobals$affylmGUIfont2))
                       Try(F2.but <- tkradiobutton(frame2,text="75%",variable=fractionTcl,value="0.75",font=.affylmGUIglobals$affylmGUIfont2))
                       Try(F3.but <- tkradiobutton(frame2,text="50%",variable=fractionTcl,value="0.5",font=.affylmGUIglobals$affylmGUIfont2))
                       Try(F4.but <- tkradiobutton(frame2,text="25%",variable=fractionTcl,value="0.25",font=.affylmGUIglobals$affylmGUIfont2))
                       Try(F5.but <- tkradiobutton(frame2,text="10%",variable=fractionTcl,value="0.1",font=.affylmGUIglobals$affylmGUIfont2))

                       Try(tkgrid(F1.but,sticky="w"))
                       Try(tkgrid(F2.but,sticky="w"))
                       Try(tkgrid(F3.but,sticky="w"))
                       Try(tkgrid(F4.but,sticky="w"))
                       Try(tkgrid(F5.but,sticky="w"))
                       Try(tkgrid.configure(fractionLabel,F1.but,F2.but,F3.but,F4.but,F5.but,sticky="w"))

                       Try(onOK <- function()
                       {
                            ReturnVal1 <- as.numeric(tclvalue(thresholdTcl))
                            ReturnVal2 <- as.numeric(tclvalue(fractionTcl))
                            Try(ReturnVal <<- paste(ReturnVal1, ReturnVal2, sep=";"))
                            Try(tkgrab.release(ttIfDialog))
                            Try(tkdestroy(ttIfDialog))
                            Try(tkfocus(.affylmGUIglobals$ttMain))
                       })

                       Try(frame3 <- tkframe(ttIfDialog,borderwidth=2))
                       Try(onCancel <- function() {Try(ReturnVal <<- ""); Try(tkgrab.release(ttIfDialog));Try(tkdestroy(ttIfDialog));Try(tkfocus(.affylmGUIglobals$ttMain))})
                       Try(OK.but <-tkbutton(frame3,text="   OK   ",command=onOK,font=.affylmGUIglobals$affylmGUIfont2))
                       Try(Cancel.but <-tkbutton(frame3,text=" Cancel ",command=onCancel,font=.affylmGUIglobals$affylmGUIfont2))

                       Try(tkgrid(tklabel(frame3,text="    "),OK.but,Cancel.but,tklabel(frame3,text="    ")))

                       Try(tkgrid(tklabel(ttIfDialog,text="    "),frame1,frame2,tklabel(ttIfDialog,text="  ")))
                       Try(tkgrid(tklabel(ttIfDialog,text="    ")))
                       Try(tkgrid(tklabel(ttIfDialog,text="    "),frame3,tklabel(ttIfDialog,text="  ")))
                       Try(tkgrid(tklabel(ttIfDialog,text="    ")))
                       Try(tkgrid.configure(frame1,frame3,sticky="w"))

                       Try(tkfocus(ttIfDialog))
                       Try(tkbind(ttIfDialog, "<Destroy>", function() {Try(tkgrab.release(ttIfDialog));Try(tkfocus(.affylmGUIglobals$ttMain));}))
                       Try(tkwait.window(ttIfDialog))
                       if(ReturnVal==""){
                            Try(tkconfigure(.affylmGUIglobals$ttMain,cursor="arrow"))
                            Try(return())
                       }
                       tmp<-strsplit(ReturnVal, ";")
                       fraction= as.numeric(tmp[[1]][2])
                       threshold= as.numeric(tmp[[1]][1])
                       tmp1 <- paste((fraction*100), "%", sep="")
                       tmp2 <- paste("threshold", as.numeric(tmp[[1]][1]), sep=" ")
                       tmp3 <- paste(tmp1, tmp2, sep=" ")
                       FilterMethod=paste("DABG filter: ", tmp3, sep=" ")
                       x <- get("NormalizedAffyData",envir=affylmGUIenvironment)
                       x.exon <- get("exonAffyData", env=affylmGUIenvironment)
                       x.p <- get("dabg.p",envir=affylmGUIenvironment)
                       
                       #f1<-pOverA(fraction, threshold)
                       #ff <- filterfun(f1)
                       #which <- genefilter(x.p, ff)
                       #selected.names <- setdiff(featureNames(x), featureNames(x)[which])
                       #selected.names <- featureNames(x)[which] #these are those data I want to remove
                       myfilter <- function(x, fraction, threshold){
                                tested <- length(which(x <= threshold))
                                ref <- length(x) * fraction
                                if(tested >= ref) return(TRUE) else return(FALSE)    
                       }
                       which <- esApply(x.p, 1, myfilter, fraction=fraction, threshold=threshold)
                       selected.names <- featureNames(x.p)[which]
                       selected.names <- intersect(featureNames(x), selected.names)
                       Try(tkmessageBox(title="DABG filter",message=paste("DABG filtering plots will be displayed in the main R window")))
                       
                       par(mfrow=c(1,2))
                       text<-paste("Unfiltered probes= ", dim(exprs(x))[1], sep="")
                       hist(as.matrix(exprs(x)), main=text, breaks=100)
                       ####temporary saving unfiltered data
                       tempUnfiltered <- tempfile(pattern = "unfiltered", tmpdir = "")
                       tempUnfiltered <- sub( "\\\\","",tempUnfiltered)
                       tempUnfiltered <- sub( "/","",tempUnfiltered)
                       workingDir <- getwd()
                       workingDir <- sub( "/$","",workingDir)  
                       tempUnfiltered <- paste(workingDir, "/", tempUnfiltered, ".rda", sep="")
                       Try(assign("unfilteredData.location", tempUnfiltered, envir=affylmGUIenvironment))
                       Try(save(x, x.exon, file=tempUnfiltered)) 
                    ####temporary saving unfiltered data

                       x <- x[which(featureNames(x)%in%selected.names),]
                       assign("NormalizedAffyData", x, envir=affylmGUIenvironment)
                       Try(x.lib <- affylmGUIenvironment$exprConsoleLibs$exon.library)
                       Try(x.lib.subset <- x.lib[which(x.lib[,1]%in%featureNames(x)),3]) #in the list obtained from NetAffx there are all the probe set ids and I have to subset only those related to the exon library loaded with the data set
                       Try(x.lib.subset <- sapply(x.lib.subset ,function(x){strsplit(x, " ")}))
                       Try(x.lib.subset<-as.vector(unlist(x.lib.subset)))
                       Try(x.exon <- x.exon[which(featureNames(x.exon)%in%x.lib.subset),])
                       Try(assign("exonAffyData", x.exon, envir=affylmGUIenvironment) )

                       text<-paste("Filtered probes= ", dim(exprs(x))[1], sep="")
                       hist(as.matrix(exprs(x)), main=text, breaks=100)
                       tkdelete(.affylmGUIglobals$mainTree,"NormalizedAffyData.Status")
                       tkinsert(.affylmGUIglobals$mainTree,"end","NormalizedAffyData","NormalizedAffyData.Status" ,text=paste("Available (",FilterMethod,")",sep=""),font=.affylmGUIglobals$affylmGUIfontTree)
                       Try(tkconfigure(.affylmGUIglobals$ttMain,cursor="arrow"))
               #######################################################

            
            
            
     } else Try(tkmessageBox(title="Filtering by DABG p-values",message="You have to calculate DABG p-values first!\n Run probesets intensity calculation by APT tools.", type="ok", icon="error"))  

}
################################################################################

"recoverUnfiltered" <- function(){
       Try(tkconfigure(.affylmGUIglobals$ttMain,cursor="watch"))
       #error if no data are loaded
       Try(whichArrayPlatform <- get("whichArrayPlatform",envir=affylmGUIenvironment))
       if(whichArrayPlatform ==""){
              Try(tkmessageBox(title="Filtering",message="No arrays have been loaded.	Please try New or Open from the File menu.",type="ok",icon="error"))
				    	Try(tkfocus(.affylmGUIglobals$ttMain))
              Try(return())
       }    
       #########################

       Try(unfilteredData.location <- get("unfilteredData.location", envir=affylmGUIenvironment))
       Try(whichArrayPlatform <- get("whichArrayPlatform", envir=affylmGUIenvironment))
       if(unfilteredData.location != "" & whichArrayPlatform=="EXON"){
            Try(load(unfilteredData.location, envir=.GlobalEnv))
            Try(assign("NormalizedAffyData", x, envir=affylmGUIenvironment))
            Try(assign("exonAffyData", x.exon, envir=affylmGUIenvironment))
            Try(tkdelete(.affylmGUIglobals$mainTree,"NormalizedAffyData.Status"))
            Try(tkinsert(.affylmGUIglobals$mainTree,"end","NormalizedAffyData","NormalizedAffyData.Status" ,text="Available",font=.affylmGUIglobals$affylmGUIfontTree))
            Try(tkdelete(.affylmGUIglobals$mainTree,"ExonArrays.Status"))
            Try(tkinsert(.affylmGUIglobals$mainTree,"end","ExonArrays","ExonArrays.Status" ,text="Available",font=.affylmGUIglobals$affylmGUIfontTree))
      #      rm(x)
      #      rm(x.exon)
      #      gc()
      #      gc()
       } else if (unfilteredData.location != ""){
            Try(load(unfilteredData.location, envir=.GlobalEnv))
            Try(assign("NormalizedAffyData", x, envir=affylmGUIenvironment))
            Try(tkdelete(.affylmGUIglobals$mainTree,"NormalizedAffyData.Status"))
            Try(tkinsert(.affylmGUIglobals$mainTree,"end","NormalizedAffyData","NormalizedAffyData.Status" ,text="Available",font=.affylmGUIglobals$affylmGUIfontTree))
       #     rm(x)
       #     gc()
       #     gc()
       } else {Try(tkmessageBox(title="Recovering unfiltered data",message="There is no need to recover unfiltered data since no filtering procedures were applied.", type="ok", icon="info"))}  

       Try(tkconfigure(.affylmGUIglobals$ttMain,cursor="arrow"))
}
################################################################################

# x is an ExpressionSet, threshold is the IQR value to be applied
"reviqrFilter"<-function(){
          #error if no data are loaded
          Try(whichArrayPlatform <- get("whichArrayPlatform",envir=affylmGUIenvironment))
          if(whichArrayPlatform !="EXON"){
              Try(tkmessageBox(title="Filtering",message="No exon array have been loaded.	Please try New or Open from the File menu.",type="ok",icon="error"))
				    	Try(tkfocus(.affylmGUIglobals$ttMain))
              Try(return())
          }    
          #########################

            require(genefilter) || stop("library genefilter could not be found !")
           	##makning the menu for selecting IQR threshold
            Try(ttGetFilterMethod <- tktoplevel(.affylmGUIglobals$ttMain))
	      Try(tkwm.deiconify(ttGetFilterMethod))
            Try(tkgrab.set(ttGetFilterMethod))
            Try(tkfocus(ttGetFilterMethod))
            Try(tkwm.title(ttGetFilterMethod,"IQR Filtering Threshold"))
	
 	      Try(tkgrid(tklabel(ttGetFilterMethod,text="    ")))
	      Try(ttGetFilterMethodTcl <- tclVar("0.8"))
            Try(rbIQR1 <- tkradiobutton(ttGetFilterMethod,text="IQR 1",variable=ttGetFilterMethodTcl,value=1,font=.affylmGUIglobals$affylmGUIfont2))
            Try(rbIQR.9<-tkradiobutton(ttGetFilterMethod,text="IQR 0.9",variable=ttGetFilterMethodTcl,value=0.9,font=.affylmGUIglobals$affylmGUIfont2))
            Try(rbIQR.8<-tkradiobutton(ttGetFilterMethod,text="IQR 0.8",variable=ttGetFilterMethodTcl,value=0.8,font=.affylmGUIglobals$affylmGUIfont2))
            Try(rbIQR.7<-tkradiobutton(ttGetFilterMethod,text="IQR 0.7",variable=ttGetFilterMethodTcl,value=0.7,font=.affylmGUIglobals$affylmGUIfont2))
            Try(rbIQR.6<-tkradiobutton(ttGetFilterMethod,text="IQR 0.6",variable=ttGetFilterMethodTcl,value=0.6,font=.affylmGUIglobals$affylmGUIfont2))
            Try(rbIQR.5<-tkradiobutton(ttGetFilterMethod,text="IQR 0.5",variable=ttGetFilterMethodTcl,value=0.5,font=.affylmGUIglobals$affylmGUIfont2))
	      Try(tkgrid(tklabel(ttGetFilterMethod,text="    "),rbIQR1))
	      Try(tkgrid(tklabel(ttGetFilterMethod,text="    "),rbIQR.9))
	      Try(tkgrid(tklabel(ttGetFilterMethod,text="    "),rbIQR.8))
	      Try(tkgrid(tklabel(ttGetFilterMethod,text="    "),rbIQR.7))
	      Try(tkgrid(tklabel(ttGetFilterMethod,text="    "),rbIQR.6))
	      Try(tkgrid(tklabel(ttGetFilterMethod,text="    "),rbIQR.5))
	      Try(tkgrid.configure(rbIQR1,rbIQR.9,rbIQR.8,rbIQR.7,rbIQR.6,rbIQR.5,columnspan=2,sticky="w"))
	      Try(tkgrid(tklabel(ttGetFilterMethod,text="    "),tklabel(ttGetFilterMethod,text="    ")))
	      Try(ReturnVal <- "")
	      Try(onCancel <- function() {Try(ReturnVal <<- "");Try(tkgrab.release(ttGetFilterMethod));Try(tkdestroy(ttGetFilterMethod));Try(tkfocus(.affylmGUIglobals$ttMain))})
	      Try(onOK <- function() {Try(ReturnVal <<- tclvalue(ttGetFilterMethodTcl));Try(tkgrab.release(ttGetFilterMethod));Try(tkdestroy(ttGetFilterMethod));Try(tkfocus(.affylmGUIglobals$ttMain))})
        Try(onHelp <- function() tkmessageBox(title="IQR reverse filtering",
                 message="IQR reverse filter removes probe set characterized by high variability \non the basis of their distribution in the arrays under analysis.\nTo be used for alternative splicing analysis only!",icon="info"))
        Try(Help.but <- tkbutton(ttGetFilterMethod,text=" Help ",command=function()Try(onHelp()),font=.affylmGUIglobals$affylmGUIfont2))
	      Try(OK.but     <- tkbutton(ttGetFilterMethod,text="OK",command=onOK,font=.affylmGUIglobals$affylmGUIfont2))
	      Try(Cancel.but <- tkbutton(ttGetFilterMethod,text="Cancel",command=onCancel,font=.affylmGUIglobals$affylmGUIfont2))

	      Try(tkgrid(tklabel(ttGetFilterMethod,text="    "),OK.but,Cancel.but, Help.but,tklabel(ttGetFilterMethod,text="    ")))
	      Try(tkgrid.configure(OK.but,sticky="e"))
	      Try(tkgrid.configure(Cancel.but,sticky="w"))
	      Try(tkgrid.configure(Help.but,sticky="e"))
	      Try(tkgrid(tklabel(ttGetFilterMethod,text="    ")))

	      Try(tkbind(ttGetFilterMethod,"<Destroy>",function() {ReturnVal <- "";Try(tkgrab.release(ttGetFilterMethod));Try(tkfocus(.affylmGUIglobals$ttMain));}))
       Try(tkbind(OK.but, "<Return>",onOK))
       Try(tkbind(Cancel.but, "<Return>",onCancel))      
        Try(tkbind(Help.but, "<Return>",onCancel)) 
	      Try(tkwait.window(ttGetFilterMethod))
        if(ReturnVal==""){
            Try(tkconfigure(.affylmGUIglobals$ttMain,cursor="arrow"))
            Try(return())
        }#cancel
        ##Filtering routine
        Try(whichArrayPlatform <- get("whichArrayPlatform", env=affylmGUIenvironment))
        FilterMethod=paste("IQR reverse filter", ReturnVal, sep=" ")
        threshold= as.numeric(ReturnVal)
        NormalizedAffyData.Available     <- get("NormalizedAffyData.Available" , envir=affylmGUIenvironment)
       Try(tkconfigure(.affylmGUIglobals$ttMain,cursor="watch"))
       x <- get("NormalizedAffyData",envir=affylmGUIenvironment)
       iqr <- function(x){IQR(x)>threshold}
       ff <- filterfun(iqr)
       which <- genefilter(x, ff)
       leftids <- setdiff(featureNames(x),featureNames(x)[which])#remaing probes after reviqrFilter
       par(mfrow=c(1,2))
       text<-paste("Unfiltered probes= ", dim(exprs(x))[1], sep="")
       hist(as.matrix(exprs(x)), breaks=100, main=text)
       text<-paste("Filtered probes= ", length(leftids), " ",FilterMethod,sep="")
       hist(as.matrix(exprs(x[which(featureNames(x)%in%leftids),])), breaks=100, main=text)
       affylmGUIenvironment$NormalizedAffyData<-x[which(featureNames(x)%in%leftids),]
       x.exon <- get("exonAffyData",envir=affylmGUIenvironment)
                            ####temporary saving unfiltered data
                            tempUnfiltered <- tempfile(pattern = "unfiltered", tmpdir = "")
                            tempUnfiltered <- sub( "\\\\","",tempUnfiltered)
                            tempUnfiltered <- sub( "/","",tempUnfiltered)
                            workingDir <- getwd()
                            workingDir <- sub( "/$","",workingDir)  
                            tempUnfiltered <- paste(workingDir, "/", tempUnfiltered, ".rda", sep="")
       assign("unfilteredData.location", tempUnfiltered, envir=affylmGUIenvironment)
       save(x, x.exon, file=tempUnfiltered)
       ####temporary saving unfiltered data
       tkdelete(.affylmGUIglobals$mainTree,"NormalizedAffyData.Status")
       tkinsert(.affylmGUIglobals$mainTree,"end","NormalizedAffyData","NormalizedAffyData.Status" ,text=paste("Available (",FilterMethod,")",sep=""),font=.affylmGUIglobals$affylmGUIfontTree)
       x <- x[which(featureNames(x)%in%leftids),]#subset of probe set to be used to extract exon data
       x.lib <- affylmGUIenvironment$exprConsoleLibs$exon.library
       x.lib.subset <- x.lib[which(x.lib[,1]%in%as.numeric(featureNames(x))),3]
       x.lib.subset <- sapply(x.lib.subset ,function(x){strsplit(x, " ")})
       x.lib.subset<-as.vector(unlist(x.lib.subset))
       affylmGUIenvironment$exonAffyData <- x.exon[which(featureNames(x.exon)%in%x.lib.subset),]
       Try(tkconfigure(.affylmGUIglobals$ttMain,cursor="arrow"))
}
################################################################################
"biomartFilter" <- function(){
           Try(tkconfigure(.affylmGUIglobals$ttMain,cursor="watch"))

           require(biomaRt) || stop("library biomaRt could not be found !")
           Try(ensembl <- useMart('ensembl'))
          #error if no data are loaded
          Try(whichArrayPlatform <- get("whichArrayPlatform",envir=affylmGUIenvironment))
          if(whichArrayPlatform ==""){
              Try(tkmessageBox(title="Filtering",message="No arrays have been loaded.	Please try New or Open from the File menu.",type="ok",icon="error"))
				    	Try(tkfocus(.affylmGUIglobals$ttMain))
              Try(return())
          }    
          if(whichArrayPlatform !="EXON"){
              Try(tkmessageBox(title="Filtering",message="No exon array have been loaded.	Please try New or Open from the File menu.",type="ok",icon="error"))
				    	Try(tkfocus(.affylmGUIglobals$ttMain))
              Try(return())
          }
          #########################
 
          Try(eg <- .gettingEG())
          Try(NormalizedAffyData <- get("NormalizedAffyData",envir=affylmGUIenvironment))
          Try(whichLib <- get("whichLib", envir=affylmGUIenvironment))
          if(whichLib[[1]][1]=="HuEx"){
                 ensembl <- useDataset('hsapiens_gene_ensembl', mart=ensembl)
          }
          if(whichLib[[1]][1]=="MoEx"){
                 ensembl <- useDataset('mmusculus_gene_ensembl', mart=ensembl)
          }
	        if(whichLib[[1]][1]=="RaEx"){
                 ensembl <- useDataset('rnorvegicus_gene_ensembl', mart=ensembl)
         }
         Try(emblInfo <- getBM(attributes = c('external_gene_id','entrezgene','ensembl_transcript_id'), filters = 'entrezgene', values = eg$EG, mart = ensembl)) 
         Try(egWithMultmRNAs <- unique(emblInfo$entrezgene[which(duplicated(emblInfo$entrezgene))]))
         Try(probesetsWithMultmRNAs <- eg$PROBESETID[which(eg$EG%in%egWithMultmRNAs)])
         Try(tkmessageBox(title="multiple mRNAs filter",message=paste("Filtering plots will be displayed in the main R window")))
         par(mfrow=c(1,2))
         text<-paste("Unfiltered probes= ", dim(exprs(NormalizedAffyData))[1], sep="")
         hist(as.matrix(exprs(NormalizedAffyData)), main=text, breaks=100)

         NormalizedAffyData <- NormalizedAffyData[which(featureNames(NormalizedAffyData)%in%probesetsWithMultmRNAs),]
         assign("NormalizedAffyData", NormalizedAffyData, envir=affylmGUIenvironment)

         Try(x.exon <- get("exonAffyData",envir=affylmGUIenvironment))
         Try(x.lib <- affylmGUIenvironment$exprConsoleLibs$exon.library)
         Try(x.lib.subset <- x.lib[which(x.lib[,1]%in%featureNames(NormalizedAffyData)),3]) #in the list obtained from NetAffx there are all the probe set ids and I have to subset only those related to the exon library loaded with the data set
         Try(x.lib.subset <- sapply(x.lib.subset ,function(x){strsplit(x, " ")}))
         Try(x.lib.subset<-as.vector(unlist(x.lib.subset)))
         Try(affylmGUIenvironment$exonAffyData <- x.exon[which(featureNames(x.exon)%in%x.lib.subset),])
         text<-paste("Filtered probes= ", dim(exprs(NormalizedAffyData))[1], sep="")
         hist(as.matrix(exprs(NormalizedAffyData)), main=text, breaks=100)
         tkdelete(.affylmGUIglobals$mainTree,"NormalizedAffyData.Status")
         tkinsert(.affylmGUIglobals$mainTree,"end","NormalizedAffyData","NormalizedAffyData.Status" ,text="Available filtered on multiple mRNAs",font=.affylmGUIglobals$affylmGUIfontTree)
         Try(tkconfigure(.affylmGUIglobals$ttMain,cursor="arrow"))
}


